"""
Library to handle .jbeam files
Support for reading, offsetting and writing nodes, with more to come.
by Tapio Kauppinen aka Taza
"""

import re


class Jbeam:
    def __init__(self, filename):
        self.__filename = filename
        self.__nodes = []
        self.__nodes_start_index = 0
        self.__nodes_end_index = 0

    def load_nodes(self, part_name):
        """
        Loads nodes to self from a part in jbeam
        :param part_name: name of the part where nodes will be loaded
        :return: int, 0 for success
        """
        jbeam_file = open(self.__filename, 'r')

        file = []
        part = []
        nodes = []

        for line in jbeam_file:
            file.append(line)
        found = False
        part_start = 0      # index where part starts in file
        for i in range(0, len(file) - 1):
            if '"' + part_name + '":{' == file[i].strip().replace(" ", ""):
                s = e = 0
                part_start = i
                for j in range(i + 1, len(file) - 1):
                    if "{" in file[j].strip():
                        s += 1
                    if "}" in file[j].strip():
                        e += 1
                    if e > s:
                        part = file[i:j + 1]
                        found = True
                        break
            if found:
                break

        # return 1 if no such part found
        if not part:
            jbeam_file.close()
            return 1

        found = False
        for i in range(0, len(part) - 1):
            if '"nodes":[' == part[i].strip().replace(" ", ""):
                s = e = 0
                for j in range(i + 1, len(part) - 1):
                    if "[" in part[j].strip():
                        s += 1
                    if "]" in part[j].strip():
                        e += 1
                    # exit when closing bracket found
                    if e > s:
                        nodes = part[i:j + 1]
                        found = True
                        self.__nodes_start_index = i + part_start
                        self.__nodes_end_index = j + 1 + part_start
                        break
            if found:
                break
        self.__nodes = nodes
        jbeam_file.close()
        return 0

    def offset_xyz(self, X, Y, Z):
        """
        Calculates new node positions based on input offsets
        :param X: offset on X axis
        :param Y: offset on Y axis
        :param Z: offset on Z axis
        :return: list, list containing changed nodes
        """
        replace = {}
        # get lines and indexes to be replaced in nodes
        for i in range(0, len(self.__nodes)-1):
            if re.search('\[".+",.+', self.__nodes[i]):  # it is valid
                # do not handle the key row
                if not self.__nodes[i].strip().startswith('["id",'):
                    # do not handle node group lists
                    if not self.__nodes[i].strip().startswith('{'):
                        replace[i] = self.__nodes[i]

        # calc new coordinates based on inputs
        for key in replace:
            split = replace[key].split(",")
            # add and round x
            split1 = split[1].strip()
            xr = max(len(str(X)[str(X).find("."):len(str(X))]),
                     len(split1[split1.find("."):len(split1)]))
            split[1] = str(round(float(split1)+X, xr))
            # add and round y
            split2 = split[2].strip()
            yr = max(len(str(Y)[str(Y).find("."):len(str(Y))]),
                     len(split2[split2.find("."):len(split2)]))
            split[2] = str(round(float(split2)+Y, yr))
            # add and round z
            split3 = split[3].strip()
            if split3.endswith("]"):
                zr = max(len(str(Z)[str(Z).find("."):len(str(Z))]),
                         len(split3[split3.find("."):len(split3)-1]))
                split[3]=str(round(float(split3[0:len(split3)-1])+Z, zr)) + "]"
            else:
                zr = max(len(str(Z)[str(Z).find("."):len(str(Z))]),
                         len(split3[split3.find("."):len(split3)]))
                split[3] = str(round(float(split3)+Z, zr))
            nodes = ", ".join(split)
            replace[key] = nodes

        # replace nodes in self, make list of changed values
        changes = ["OLD NODES:"]
        for line in self.__nodes:
            changes.append(line.strip())
        for key in replace:
            self.__nodes[key] = replace[key]
        changes.append("NEW NODES:")
        for line in self.__nodes:
            changes.append(line.strip())
        return changes

    def write_nodes(self):
        """
        Writes nodes from self to file
        :return: None
        """
        # no checks if file exists, meant to run after reading from file
        jbeam_file = open(self.__filename, 'r')

        # read whole file
        file = []
        for line in jbeam_file:
            file.append(line)
        jbeam_file.close()

        # replace file nodes with self nodes
        j = 0
        for i in range(self.__nodes_start_index, self.__nodes_end_index):
            file[i] = self.__nodes[j]
            j += 1

        # write file
        jbeam_file = open(self.__filename, 'w')
        for line in file:
            jbeam_file.write(line)
        jbeam_file.close()
